'use strict';

//The unit test ('g' = global)
let gUnitTest = null;

//When doc is ready, then call "buildController" 
document.onreadystatechange = function () {
	if(document.readyState === 'complete') {
		attachEvents();
		buildTestCaseList();		
		//buildSubTestList();
		changedSubTests();
	}
}

function attachEvents() {
	//actions
	//elem('runAll').addEventListener('click', cRunAll, false);
	elem('runSelected').addEventListener('click', cRunSelected, false);	
	elem('selectAll').addEventListener('click', cSelectAll, false);		
	
	//elem('clear').addEventListener('click', cClear, false);		
	//changes
	elem('subTests').addEventListener('input', changedSubTests, false);	
	elem('testCases').addEventListener('input', changedTestCases, false);
}

/*function cRunAll() {
	gUnitTest = newTestCase();
	UnitTestManager.runUnitTest(gUnitTest);
	showOutput();
}*/

function changedSubTests() {
	cClear();
	elem('runSelected').disabled = !hasSelection(elem('subTests'));
}

function cRunSelected() {
	const selections = selectedItemsFrom(elem('subTests'));
	gUnitTest = newTestCase();		
	gUnitTest = UnitTestManager.runUnitTestWith(gUnitTest, selections);
	showOutput();
}

function cSelectAll() {
	selectAllTests();
}

function cClear() {
	if (gUnitTest)
		gUnitTest.clear();
	showOutput();
	clearStatus();
}

function clearStatus() {
	elem('status').value = null;
	setColors('status', null, null);	
}

function selectedItemsFrom(selectField) {
	const result = [];
	for (let opt of selectField.options)
		if (opt.selected)
			result.push(opt.value);
	return result;
}

function hasSelection(selectField) {
	for (let opt of selectField.options)
		if (opt.selected)
			return true;
	return false;
}

function elem(nm) {
	//Return field named 'nm'	
	return document.getElementById(nm);
}	

function setLog(str) {
	elem('testLog').innerHTML = str;
}

function showOutput() {
	if (!gUnitTest) return;
	elem('testLog').value = gUnitTest.getLog();
	if (gUnitTest.isPassing())
		setColors('status', 'white', 'green');
	else
		setColors('status', 'white', 'red');
	elem('status').value = gUnitTest.statusLabel();
}

function setColors(id, fore, back) {
	let field = elem(id);
	field.style.backgroundColor = back;
	field.style.color = fore;
}


function buildSubTestList() {
	const list = elem('subTests');
	clearTestsOptions();
	newTestCase().subTests().forEach(subTest => {
		let item, testSelector;
		item = document.createElement("option");
		testSelector = subTest.getTestLabel();
		//TODO -- getTestLabel should be getTestName
		item.value = testSelector;
		item.innerText = testSelector;
		list.appendChild(item);	
	});
	selectAllTests();	
}

function selectAllTests() {
	const list = elem('subTests');
	Array.from(list.options).forEach(opt => opt.selected = true);
}

//----------------------------------------------------
//Test Cases

function removeAllChildrenElements(parentElem) {
    while (parentElem.lastChild)
        parentElem.removeChild(parentElem.lastChild);
};

function clearTestsOptions() {
	//removeAllChildrenElements(list);
	//(Array.from(list.options)).forEach(opt => opt.remove());
	//list.innerHTML = '';
	const tests = elem('subTests');	
	//removeAllChildrenElements(tests);
	tests.innerHTML = '';	
}

function buildTestCaseList() {	
	assureCurrentTestCase();
	const list = elem('testCases');
	testClassClasses.forEach(testCaseClassName => {
		let item;
		item = document.createElement("option");
		item.value = testCaseClassName;
		item.innerText = testCaseClassName;
		list.appendChild(item);	
	});
	if (list.options.length > 0) {
		list.options[0].selected = true;
		changedTestCases();
	}
}

function changedTestCases() {
	const testCaseClassName = elem('testCases').value;
	if (!testCaseClassName) return;
	cClear();
	setCurrentTestCase(testCaseClassName);
	buildSubTestList();
}

