'use strict';

class Test {
    /* unitTest
    testLabel
    testFunction*/

constructor(unitTest, testLabel, testFunction) {
        this.unitTest = unitTest;
        this.testLabel = testLabel;
        this.testFunction = testFunction;
    }
	
    getTestFunction() {
        return this.testFunction;
    }

    getTestLabel() {
        return this.testLabel;
    }

    run() {
        this.unitTest.show("Running -- " + this.testLabel);
        this.unitTest.setCurrentSubTest(this);
        this.testFunction();
        this.unitTest.setCurrentSubTest(null);
    }

    toString() {
        return this.testLabel;
    }
}


//==================================================================





class TestRunner {
    /*unitTest
    subTests*/

    constructor(unitTest, subTests) {
        this.unitTest = unitTest;
        this.subTests = subTests;
    }
	
	preCheck() {
		const preTestError = this.unitTest.preTest();		
		if (!preTestError)
			return true;
        const utest = this.unitTest;
		utest
			.show('PRE-CHECK FAILED')
			.show(`Error: ${preTestError}`)
			.show('Could not run tests')		
			.incrementExceptionCount();
		return false;
	}
		
    runAll() {
		if (!this.preCheck()) return;
        const utest = this.unitTest;		
        utest.beforeAll();
        for (let eachSubTest of this.subTests) {
			
            //utest.beforeEach();
            this.runSubTest(eachSubTest, utest);
            //utest.afterEach();
        }
        utest.afterAll();
        this.unitTest.show("");
		const uname = utest.getUnitName();
		if (!isString(uname))
			name = uname.name;
        utest.printSummary(uname);
    }

    runSubTest(subTest, utest) {
        //xdoc method cascading
        try {
			utest.beforeEach();
            subTest.run();
            utest.afterEach();			
        } catch (ex) {
			const ut = this.unitTest;
            ut
				.incrementExceptionCount()
				.show(`EXCEPTION OCCURRED in ${subTest.toString()}`);
			exceptionToDisplayStrings(ex).forEach(each => ut.show(each));
			ut.show('');
		}
	}
	
	//--------------------------------------------------------
		
}

//==================================================================

class UnitTestManager {

    /*unitTest;
    subTests*/

    constructor(testee, subTests) {
        this.unitTest = testee;
        this.subTests = subTests;
    }

    getUnitTest() {
        return this.unitTest;
    }

    getSubTests() {
        return this.subTests;
    }

    testRunner() {
        return new TestRunner(this.unitTest, this.subTests);
    }

    runAll() {
        this.testRunner().runAll();
    }

    static runUnitTest(unitTest) {
        let mgr;
        mgr = new UnitTestManager(unitTest, unitTest.subTests());
        mgr.runAll();
        return unitTest;
    }
	
    static runUnitTestWith(unitTest, subTestNames) {
        let mgr, subTests;
		subTests = unitTest.subTests();
		subTests = subTests.filter(eaSubTest => subTestNames.includes(eaSubTest.getTestLabel()));
        mgr = new UnitTestManager(unitTest, subTests);
        mgr.runAll();
        return unitTest;
    }	

}

//***************************************************
//***************************************************

//----------------------------------------------------
//Helpers

function prn(anObject) {
    let safeValue = anObject;
    if (safeValue === undefined)
        safeValue = '*undefined*';
    if (safeValue === null)
        safeValue = '*null*';
    console.log(safeValue.toString());
}
